import { Vector } from '../vector.js';
import { TypeMap } from '../type.js';
import { Schema } from '../schema.js';
import { RecordBatch } from '../recordbatch.js';
/** @ignore */
export declare function distributeVectorsIntoRecordBatches<T extends TypeMap = any>(schema: Schema<T>, vecs: Vector<T[keyof T]>[]): [Schema<T>, RecordBatch<T>[]];
