/**
 * Converts an integer as a number or bigint to a number, throwing an error if the input cannot safely be represented as a number.
 */
export declare function bigIntToNumber(number: bigint | number): number;
