import { Bool } from '../type.js';
import { Builder, BuilderOptions } from '../builder.js';
/** @ignore */
export declare class BoolBuilder<TNull = any> extends Builder<Bool, TNull> {
    constructor(options: BuilderOptions<Bool, TNull>);
    setValue(index: number, value: boolean): void;
}
